
class Neopet:
    """Represents a Neopet

    Attributes:
        | **name**: The neopet's name
        | **gender**: The neopet's gender
        | **species**: The neopet's species
        | **age**: The neopet's age
        | **level**: The neopet's level

        | **health**: The neopet's current health
        | **mood**: The neopet's current mood
        | **hunger**: The neopet's current hunger level
    """
    name = ''
    gender = ''
    species = ''
    age = 0
    level = 0

    health = ''
    mood = ''
    hunger = ''
